@@ -28,7 +28,7 @@
 G_BEGIN_DECLS
 
 typedef struct _GIrTypelibBuild GIrTypelibBuild;
-typedef struct _GIrNode GIrNode; 
+typedef struct _GIrNode GIrNode;
 typedef struct _GIrNodeFunction GIrNodeFunction;
 typedef struct _GIrNodeParam GIrNodeParam;
 typedef struct _GIrNodeType GIrNodeType;
@@ -51,12 +51,12 @@ struct _GIrTypelibBuild {
   GList       *modules;
   GHashTable  *strings;
   GHashTable  *types;
-  GList       *offset_ordered_nodes;
+  GList       *nodes_with_attributes;
   guint32      n_attributes;
-  guchar      *data; 
+  guchar      *data;
 };
 
-typedef enum 
+typedef enum
 {
   G_IR_NODE_INVALID      =  0,
   G_IR_NODE_FUNCTION     =  1,
@@ -64,7 +64,7 @@ typedef enum
   G_IR_NODE_STRUCT       =  3,
   G_IR_NODE_BOXED        =  4,
   G_IR_NODE_ENUM         =  5,
-  G_IR_NODE_FLAGS        =  6, 
+  G_IR_NODE_FLAGS        =  6,
   G_IR_NODE_OBJECT       =  7,
   G_IR_NODE_INTERFACE    =  8,
   G_IR_NODE_CONSTANT     =  9,
@@ -102,7 +102,7 @@ struct _GIrNodeFunction
   GIrNode node;
 
   gboolean deprecated;
-  gboolean is_varargs; /* Not in typelib yet */ 
+  gboolean is_varargs; /* Not in typelib yet */
 
   gboolean is_method;
   gboolean is_setter;
@@ -117,7 +117,7 @@ struct _GIrNodeFunction
   GList *parameters;
 };
 
-struct _GIrNodeType 
+struct _GIrNodeType
 {
   GIrNode node;
 
@@ -138,31 +138,32 @@ struct _GIrNodeType
   gint length;
   gboolean has_size;
   gint size;
-  
+  gint array_type;
+
   GIrNodeType *parameter_type1;
-  GIrNodeType *parameter_type2;  
+  GIrNodeType *parameter_type2;
 
   gchar *interface;
   gchar **errors;
 };
 
-struct _GIrNodeParam 
+struct _GIrNodeParam
 {
   GIrNode node;
 
   gboolean in;
   gboolean out;
-  gboolean dipper;
+  gboolean caller_allocates;
   gboolean optional;
   gboolean retval;
   gboolean allow_none;
   gboolean transfer;
   gboolean shallow_transfer;
   GIScopeType scope;
-  
+
   gint8 closure;
   gint8 destroy;
-  
+
   GIrNodeType *type;
 };
 
@@ -177,11 +178,13 @@ struct _GIrNodeProperty
   gboolean writable;
   gboolean construct;
   gboolean construct_only;
-  
+  gboolean transfer;
+  gboolean shallow_transfer;
+
   GIrNodeType *type;
 };
 
-struct _GIrNodeSignal 
+struct _GIrNodeSignal
 {
   GIrNode node;
 
@@ -194,30 +197,30 @@ struct _GIrNodeSignal
   gboolean detailed;
   gboolean action;
   gboolean no_hooks;
-  
+
   gboolean has_class_closure;
   gboolean true_stops_emit;
-  
+
   gint class_closure;
-  
+
   GList *parameters;
-  GIrNodeParam *result;    
+  GIrNodeParam *result;
 };
 
-struct _GIrNodeVFunc 
+struct _GIrNodeVFunc
 {
   GIrNode node;
 
-  gboolean is_varargs; /* Not in typelib yet */ 
+  gboolean is_varargs; /* Not in typelib yet */
   gboolean must_chain_up;
   gboolean must_be_implemented;
   gboolean must_not_be_implemented;
   gboolean is_class_closure;
-  
+
   char *invoker;
 
   GList *parameters;
-  GIrNodeParam *result;      
+  GIrNodeParam *result;
 
   gint offset;
 };
@@ -231,7 +234,7 @@ struct _GIrNodeField
   gint bits;
   gint offset;
   GIrNodeFunction *callback;
-  
+
   GIrNodeType *type;
 };
 
@@ -241,19 +244,25 @@ struct _GIrNodeInterface
 
   gboolean abstract;
   gboolean deprecated;
+  gboolean fundamental;
 
   gchar *gtype_name;
   gchar *gtype_init;
 
+  gchar *ref_func;
+  gchar *unref_func;
+  gchar *set_value_func;
+  gchar *get_value_func;
+
   gchar *parent;
   gchar *glib_type_struct;
-  
+
   GList *interfaces;
   GList *prerequisites;
 
   gint alignment;
   gint size;
-  
+
   GList *members;
 };
 
@@ -273,7 +282,7 @@ struct _GIrNodeConstant
   gboolean deprecated;
 
   GIrNodeType *type;
-  
+
   gchar *value;
 };
 
@@ -291,7 +300,7 @@ struct _GIrNodeEnum
 };
 
 struct _GIrNodeBoxed
-{ 
+{
   GIrNode node;
 
   gboolean deprecated;
@@ -301,7 +310,7 @@ struct _GIrNodeBoxed
 
   gint alignment;
   gint size;
-  
+
   GList *members;
 };
 
@@ -312,13 +321,14 @@ struct _GIrNodeStruct
   gboolean deprecated;
   gboolean disguised;
   gboolean is_gtype_struct;
+  gboolean foreign;
 
   gchar *gtype_name;
   gchar *gtype_init;
 
   gint alignment;
   gint size;
-  
+
   GList *members;
 };
 
@@ -327,7 +337,7 @@ struct _GIrNodeUnion
   GIrNode node;
 
   gboolean deprecated;
-  
+
   GList *members;
   GList *discriminators;
 
@@ -347,7 +357,7 @@ struct _GIrNodeErrorDomain
   GIrNode node;
 
   gboolean deprecated;
-  
+
   gchar *name;
   gchar *getquark;
   gchar *codes;
@@ -358,7 +368,6 @@ GIrNode * g_ir_node_new             (GIrNodeTypeId type);
 void      g_ir_node_free            (GIrNode    *node);
 guint32   g_ir_node_get_size        (GIrNode    *node);
 guint32   g_ir_node_get_full_size   (GIrNode    *node);
-guint32   g_ir_node_get_attribute_size (GIrNode *node);
 void      g_ir_node_build_typelib   (GIrNode         *node,
                                      GIrNode         *parent,
                                      GIrTypelibBuild *build,
@@ -370,7 +379,7 @@ gboolean  g_ir_node_can_have_member (GIrNode    *node);
 void      g_ir_node_add_member      (GIrNode         *node,
 				     GIrNodeFunction *member);
 guint32   write_string              (const gchar *str,
-				     GHashTable  *strings, 
+				     GHashTable  *strings,
 				     guchar      *data,
 				     guint32     *offset);
 
