@@ -76,6 +76,7 @@ typedef enum
   STATE_ALIAS, /* 30 */
   STATE_TYPE,
   STATE_ATTRIBUTE,
+  STATE_DOC,
   STATE_PASSTHROUGH
 } ParseState;
 
@@ -325,19 +326,38 @@ push_node (ParseContext *ctx, GIrNode *node)
 static GIrNodeType * parse_type_internal (const gchar *str, gchar **next, gboolean in_glib,
 					  gboolean in_gobject);
 
+typedef struct {
+  const gchar *str;
+  guint size;
+  guint is_signed : 1;
+} IntegerAliasInfo;
+
+static IntegerAliasInfo integer_aliases[] = {
+  { "char",     SIZEOF_CHAR,      0 },
+  { "short",    SIZEOF_SHORT,     1 },
+  { "ushort",   SIZEOF_SHORT,     0 },
+  { "int",      SIZEOF_INT,       1 },
+  { "uint",     SIZEOF_INT,       0 },
+  { "long",     SIZEOF_LONG,      1 },
+  { "ulong",    SIZEOF_LONG,      0 },
+  { "gsize",    GLIB_SIZEOF_SIZE_T,    0 },
+  { "gssize",   GLIB_SIZEOF_SIZE_T,    1 },
+};
+
 typedef struct {
   const gchar *str;
   gint tag;
   gboolean pointer;
 } BasicTypeInfo;
 
+#define BASIC_TYPE_FIXED_OFFSET 3
+
 static BasicTypeInfo basic_types[] = {
     { "none",     GI_TYPE_TAG_VOID,    0 },
     { "any",      GI_TYPE_TAG_VOID,    1 },
 
     { "bool",     GI_TYPE_TAG_BOOLEAN, 0 },
-    { "char",     GI_TYPE_TAG_INT8,    0 },
-    { "int8",     GI_TYPE_TAG_INT8,    0 },
+    { "int8",     GI_TYPE_TAG_INT8,    0 }, /* Start of BASIC_TYPE_FIXED_OFFSET */
     { "uint8",    GI_TYPE_TAG_UINT8,   0 },
     { "int16",    GI_TYPE_TAG_INT16,   0 },
     { "uint16",   GI_TYPE_TAG_UINT16,  0 },
@@ -345,19 +365,8 @@ static BasicTypeInfo basic_types[] = {
     { "uint32",   GI_TYPE_TAG_UINT32,  0 },
     { "int64",    GI_TYPE_TAG_INT64,   0 },
     { "uint64",   GI_TYPE_TAG_UINT64,  0 },
-    { "short",    GI_TYPE_TAG_SHORT,   0 },
-    { "ushort",   GI_TYPE_TAG_USHORT,  0 },
-    { "int",      GI_TYPE_TAG_INT,     0 },
-    { "uint",     GI_TYPE_TAG_UINT,    0 },
-    { "long",     GI_TYPE_TAG_LONG,    0 },
-    { "ulong",    GI_TYPE_TAG_ULONG,   0 },
-    { "ssize_t",  GI_TYPE_TAG_SSIZE,   0 },
-    { "ssize",    GI_TYPE_TAG_SSIZE,   0 },
-    { "size_t",   GI_TYPE_TAG_SIZE,    0 },
-    { "size",     GI_TYPE_TAG_SIZE,    0 },
     { "float",    GI_TYPE_TAG_FLOAT,   0 },
     { "double",   GI_TYPE_TAG_DOUBLE,  0 },
-    { "time_t",   GI_TYPE_TAG_TIME_T,  0 },
     { "GType",    GI_TYPE_TAG_GTYPE,   0 },
     { "utf8",     GI_TYPE_TAG_UTF8,    1 },
     { "filename", GI_TYPE_TAG_FILENAME,1 },
@@ -374,6 +383,41 @@ parse_basic (const char *str)
       if (g_str_has_prefix (str, basic_types[i].str))
 	return &(basic_types[i]);
     }
+  for (i = 0; i < G_N_ELEMENTS (integer_aliases); i++)
+    {
+      if (g_str_has_prefix (str, integer_aliases[i].str))
+	{
+	  switch (integer_aliases[i].size)
+	    {
+	    case sizeof(guint8):
+	      if (integer_aliases[i].is_signed)
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET];
+	      else
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+1];
+	      break;
+	    case sizeof(guint16):
+	      if (integer_aliases[i].is_signed)
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+2];
+	      else
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+3];
+	      break;
+	    case sizeof(guint32):
+	      if (integer_aliases[i].is_signed)
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+4];
+	      else
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+5];
+	      break;
+	    case sizeof(guint64):
+	      if (integer_aliases[i].is_signed)
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+6];
+	      else
+		return &basic_types[BASIC_TYPE_FIXED_OFFSET+7];
+	      break;
+	    default:
+	      g_assert_not_reached ();
+	    }
+	}
+    }
   return NULL;
 }
 
@@ -1639,6 +1683,11 @@ start_class (GMarkupParseContext *context,
   const gchar *typeinit;
   const gchar *deprecated;
   const gchar *abstract;
+  const gchar *fundamental;
+  const gchar *ref_func;
+  const gchar *unref_func;
+  const gchar *set_value_func;
+  const gchar *get_value_func;
   GIrNodeInterface *iface;
 
   if (!(strcmp (element_name, "class") == 0 &&
@@ -1655,6 +1704,11 @@ start_class (GMarkupParseContext *context,
   typeinit = find_attribute ("glib:get-type", attribute_names, attribute_values);
   deprecated = find_attribute ("deprecated", attribute_names, attribute_values);
   abstract = find_attribute ("abstract", attribute_names, attribute_values);
+  fundamental = find_attribute ("glib:fundamental", attribute_names, attribute_values);
+  ref_func = find_attribute ("glib:ref-func", attribute_names, attribute_values);
+  unref_func = find_attribute ("glib:unref-func", attribute_names, attribute_values);
+  set_value_func = find_attribute ("glib:set-value-func", attribute_names, attribute_values);
+  get_value_func = find_attribute ("glib:get-value-func", attribute_names, attribute_values);
 
   if (name == NULL)
     {
@@ -1685,6 +1739,15 @@ start_class (GMarkupParseContext *context,
 
   iface->abstract = abstract && strcmp (abstract, "1") == 0;
 
+  if (ref_func)
+    iface->ref_func = g_strdup (ref_func);
+  if (unref_func)
+    iface->unref_func = g_strdup (unref_func);
+  if (set_value_func)
+    iface->set_value_func = g_strdup (set_value_func);
+  if (get_value_func)
+    iface->get_value_func = g_strdup (get_value_func);
+
   push_node (ctx, (GIrNode *) iface);
   ctx->current_module->entries =
     g_list_append (ctx->current_module->entries, iface);
@@ -1957,6 +2020,22 @@ end_type (ParseContext *ctx)
     }
 }
 
+static gboolean
+start_doc (GMarkupParseContext *context,
+	   const gchar         *element_name,
+	   const gchar        **attribute_names,
+	   const gchar        **attribute_values,
+	   ParseContext       *ctx,
+	   GError             **error)
+{
+  if (strcmp (element_name, "doc") != 0 || ctx->node_stack == NULL)
+    return FALSE;
+
+  state_switch (ctx, STATE_DOC);
+
+  return TRUE;
+}
+
 static gboolean
 start_attribute (GMarkupParseContext *context,
                  const gchar         *element_name,
@@ -2570,6 +2649,9 @@ start_element_handler (GMarkupParseContext *context,
 			       attribute_names, attribute_values,
 			       ctx, error))
 	goto out;
+      else if (start_doc (context, element_name, attribute_names,
+			  attribute_values, ctx, error))
+	goto out;
       break;
 
     case 'e':
@@ -3207,6 +3289,13 @@ end_element_handler (GMarkupParseContext *context,
         }
       break;
 
+    case STATE_DOC:
+      if (strcmp ("doc", element_name) == 0)
+        {
+          state_switch (ctx, ctx->prev_state);
+        }
+      break;
+
     case STATE_PASSTHROUGH:
       ctx->unknown_depth -= 1;
       if (ctx->unknown_depth == 0)
