@@ -20,6 +20,6 @@
 
 DATADIR = "${prefix}/share"
 DATADIR = DATADIR.replace(
-    "${prefix}", "/home/jdahlin/.local")
-GIR_DIR = "/home/jdahlin/.local/share/gir-1.0"
+    "${prefix}", "/home/desrt/local")
+GIR_DIR = "/home/desrt/local/share/gir-1.0"
 GIR_SUFFIX = "gir-1.0"
