@@ -18,6 +18,8 @@
 # Boston, MA 02111-1307, USA.
 #
 
+import os
+
 from xml.etree.cElementTree import parse
 
 from .ast import (Alias, Array, Callback, Constant, Enum, Function, Field,
@@ -27,6 +29,8 @@ from .glibast import (GLibEnum, GLibEnumMember, GLibFlags,
                       GLibInterface, GLibObject, GLibBoxedStruct,
                       GLibBoxedUnion, GLibBoxedOther)
 
+from .girwriter import COMPATIBLE_GIR_VERSION
+
 CORE_NS = "http://www.gtk.org/introspection/core/1.0"
 C_NS = "http://www.gtk.org/introspection/c/1.0"
 GLIB_NS = "http://www.gtk.org/introspection/glib/1.0"
@@ -52,12 +56,16 @@ class GIRParser(object):
         self._includes = set()
         self._pkgconfig_packages = set()
         self._namespace = None
+        self._filename_stack = []
 
     # Public API
 
     def parse(self, filename):
+        filename = os.path.abspath(filename)
+        self._filename_stack.append(filename)
         tree = parse(filename)
         self.parse_tree(tree)
+        self._filename_stack.pop()
 
     def parse_tree(self, tree):
         self._includes.clear()
@@ -88,11 +96,26 @@ class GIRParser(object):
 
     # Private
 
+    def _get_current_file(self):
+        if not self._filename_stack:
+            return None
+        cwd = os.getcwd() + os.sep
+        curfile = self._filename_stack[-1]
+        if curfile.startswith(cwd):
+            return curfile[len(cwd):]
+        return curfile
+
     def _add_node(self, node):
         self._namespace.nodes.append(node)
 
     def _parse_api(self, root):
         assert root.tag == _corens('repository')
+        version = root.attrib['version']
+        if version != COMPATIBLE_GIR_VERSION:
+            raise SystemExit("%s: Incompatible version %s (supported: %s)" \
+                             % (self._get_current_file(),
+                                version, COMPATIBLE_GIR_VERSION))
+
         for node in root.getchildren():
             if node.tag == _corens('include'):
                 self._parse_include(node)
