@@ -18,13 +18,14 @@
 # Boston, MA 02111-1307, USA.
 #
 
-from .ast import Class, Enum, Interface, Member, Node, Property, Struct, Union
+from .ast import (Bitfield, Class, Enum, Interface, Member, Node,
+                  Property, Struct, Union, Record)
 from .ast import (
     type_names, default_array_types,
-    TYPE_STRING, TYPE_INT8, TYPE_UINT8, TYPE_INT16, TYPE_UINT16,
-    TYPE_INT, TYPE_UINT, TYPE_UINT32, TYPE_INT32, TYPE_LONG,
-    TYPE_ULONG, TYPE_INT64, TYPE_UINT64, TYPE_FLOAT,
-    TYPE_DOUBLE, TYPE_BOOLEAN, TYPE_ANY, TYPE_SSIZET,
+    TYPE_STRING, TYPE_INT8, TYPE_UINT8, TYPE_SHORT, TYPE_USHORT,
+    TYPE_INT16, TYPE_UINT16, TYPE_INT, TYPE_UINT, TYPE_UINT32,
+    TYPE_INT32, TYPE_LONG, TYPE_ULONG, TYPE_INT64, TYPE_UINT64,
+    TYPE_FLOAT, TYPE_DOUBLE, TYPE_BOOLEAN, TYPE_ANY, TYPE_SSIZET,
     TYPE_SIZET)
 
 
@@ -52,8 +53,8 @@ type_names['gsize'] = TYPE_SIZET
 type_names['gssize'] = TYPE_SSIZET
 type_names['gchar'] = TYPE_INT8
 type_names['guchar'] = TYPE_UINT8
-type_names['gshort'] = TYPE_INT16
-type_names['gushort'] = TYPE_UINT16
+type_names['gshort'] = TYPE_SHORT
+type_names['gushort'] = TYPE_USHORT
 
 # It's not very nice to duplicate the array types from ast.py,
 # but a clean fix is hard without essentially hardcoding
@@ -61,6 +62,22 @@ type_names['gushort'] = TYPE_UINT16
 default_array_types['guint8*'] = TYPE_UINT8
 default_array_types['gchar**'] = TYPE_STRING
 
+class GLibRecord(Record):
+    def __init__(self, *args, **kwargs):
+        Record.__init__(self, *args, **kwargs)
+
+    @classmethod
+    def from_record(cls, record):
+        obj = cls(record.name, record.symbol)
+        obj.fields = record.fields
+        obj.constructors = record.constructors
+        obj.disguised = record.disguised
+        obj.doc = record.doc
+        obj.methods = record.methods
+        # If true, this record defines the FooClass C structure
+        # for some Foo GObject (or similar for GInterface)
+        obj.is_gtype_struct_for = False
+        return obj
 
 class GLibEnum(Enum):
 
@@ -69,17 +86,24 @@ class GLibEnum(Enum):
         self.ctype = type_name
         self.type_name = type_name
         self.get_type = get_type
+        self.error_quark = None
 
     def __repr__(self):
-        return '%s(%r, %r, %r)' % (
-            self.__class__.__name__,
-            self.name,
-            self.members,
-            self.get_type)
+        return 'GlibEnum(%r, %r, %r)' % (self.name, self.members,
+                                         self.get_type)
 
 
-class GLibFlags(GLibEnum):
-    pass
+class GLibFlags(Bitfield):
+
+    def __init__(self, name, type_name, members, get_type):
+        Bitfield.__init__(self, name, type_name, members)
+        self.ctype = type_name
+        self.type_name = type_name
+        self.get_type = get_type
+
+    def __repr__(self):
+        return 'GlibFlags(%r, %r, %r)' % (self.name, self.members,
+                                          self.get_type)
 
 
 class GLibEnumMember(Member):
@@ -96,6 +120,11 @@ class GLibObject(Class):
         Class.__init__(self, name, parent, is_abstract)
         self.type_name = type_name
         self.get_type = get_type
+        self.fundamental = False
+        self.unref_func = None
+        self.ref_func = None
+        self.set_value_func = None
+        self.get_value_func = None
         self.signals = []
         self.ctype = ctype or type_name
 
@@ -103,8 +132,6 @@ class GLibObject(Class):
 class GLibBoxed:
 
     def __init__(self, type_name, get_type):
-        self.constructors = []
-        self.methods = []
         self.type_name = type_name
         self.get_type = get_type
 
@@ -128,8 +155,10 @@ class GLibBoxedOther(Node, GLibBoxed):
     def __init__(self, name, type_name, get_type):
         Node.__init__(self, name)
         GLibBoxed.__init__(self, type_name, get_type)
+        self.constructors = []
+        self.methods = []
         self.ctype = type_name
-
+        self.doc = None
 
 class GLibInterface(Interface):
 
@@ -152,3 +181,4 @@ class GLibSignal(Node):
         Node.__init__(self, name)
         self.retval = retval
         self.parameters = []
+        self.doc = None
